@@ -3,13 +3,13 @@ import logging
 import math
 import random
 import sys
-import time
+
 from collections import defaultdict
 from functools import lru_cache
 from typing import Dict, Union
-
+from xml.etree import cElementTree as ET
 import numpy as np
-import sumocr
+
 from commonroad.geometry.shape import Rectangle
 from commonroad.planning.planning_problem import PlanningProblemSet
 from commonroad.common.util import Interval
@@ -17,32 +17,23 @@ from commonroad.prediction.prediction import TrajectoryPrediction
 from commonroad.scenario.lanelet import LaneletNetwork
 from commonroad.scenario.obstacle import DynamicObstacle
 from commonroad.scenario.scenario import Scenario
-from commonroad.scenario.trajectory import State, Trajectory
+from commonroad.scenario.state import CustomState, State
+from commonroad.scenario.trajectory import Trajectory
 
+import sumocr
 from sumocr.interface.ego_vehicle import EgoVehicle
 from sumocr.interface.util import *
 from sumocr.maps.scenario_wrapper import AbstractScenarioWrapper
 from sumocr.maps.sumo_scenario import ScenarioWrapper
 from sumocr.sumo_config import DefaultConfig, EGO_ID_START, ID_DICT
 from sumocr.sumo_config.pathConfig import SUMO_BINARY, SUMO_GUI_BINARY
-from xml.etree import cElementTree as ET
 
-try:
-    import traci
-    from traci._person import PersonDomain
-    from traci._vehicle import VehicleDomain
-    from traci._simulation import SimulationDomain
-    from traci._route import RouteDomain
-except ModuleNotFoundError as exp:
-    PersonDomain = DummyClass
-    VehicleDomain = DummyClass
-    SimulationDomain = DummyClass
-    RouteDomain = DummyClass
+import libsumo as traci
 
 __author__ = "Moritz Klischat, Maximilian Frühauf"
 __copyright__ = "TUM Cyber-Physical Systems Group"
 __credits__ = ["ZIM Projekt ZF4086007BZ8"]
-__version__ = "2021.4"
+__version__ = "2022.1"
 __maintainer__ = "Moritz Klischat"
 __email__ = "commonroad@lists.lrz.de"
 __status__ = "Released"
@@ -61,10 +52,10 @@ class SumoSimulation:
         self.planning_problem_set: PlanningProblemSet = None
         # {time_step: {obstacle_id: state}}
         self.obstacle_states: Dict[int, Dict[int, State]] = defaultdict(lambda: dict())
-        self.simulationdomain: SimulationDomain = SimulationDomain()
-        self.vehicledomain: VehicleDomain = VehicleDomain()
-        self.persondomain: PersonDomain = PersonDomain()
-        self.routedomain: RouteDomain = RouteDomain()
+        self.simulationdomain = traci.simulation
+        self.vehicledomain = traci.vehicle
+        self.persondomain = traci.person
+        self.routedomain = traci.route
         self._current_time_step = 0
         self.ids_sumo2cr, self.ids_cr2sumo = initialize_id_dicts(ID_DICT)
         self._max_lanelet_network_id = 0 # keep track of all IDs in CR lanelet_network
@@ -94,7 +85,7 @@ class SumoSimulation:
         return self._scenarios
 
     @scenarios.setter
-    def scenarios(self, scenarios: AbstractScenarioWrapper):
+    def scenarios(self, scenarios: ScenarioWrapper):
         def max_lanelet_network_id(lanelet_network: LaneletNetwork) -> int:
             max_lanelet = np.max([l.lanelet_id for l in lanelet_network.lanelets]) \
                 if lanelet_network.lanelets else 0
@@ -120,7 +111,7 @@ class SumoSimulation:
         self._scenarios = scenarios
 
     def initialize(self, conf: DefaultConfig,
-                   scenario_wrapper: AbstractScenarioWrapper,
+                   scenario_wrapper: ScenarioWrapper,
                    planning_problem_set: PlanningProblemSet = None) -> None:
         """
         Reads scenario files, starts traci simulation, initializes vehicles, conducts pre-simulation.
@@ -137,8 +128,8 @@ class SumoSimulation:
 
         self.logger = self._init_logging()
 
-        # assert isinstance(scenario_wrapper, AbstractScenarioWrapper), \
-        #     f'scenario_wrapper expected type AbstractScenarioWrapper or None, but got type {type(scenario_wrapper)}'
+        assert isinstance(scenario_wrapper, AbstractScenarioWrapper), \
+            f'scenario_wrapper expected type ScenarioWrapper or None, but got type {type(scenario_wrapper)}'
         self.scenarios = scenario_wrapper
         self.dt = self.conf.dt
         self.dt_sumo = self.conf.dt / self.conf.delta_steps
@@ -173,10 +164,10 @@ class SumoSimulation:
             random.seed(self.conf.random_seed)
             cmd.extend(['--seed', str(self.conf.random_seed)])
 
-        traci.start(cmd, label=self.traci_label)
+        traci.start(cmd)
         # simulate until ego_time_start
         if self.planning_problem_set is not None and len(self._find_ego_vehicles_in_rou_file()) == 0:
-            self.__presimulation_silent(self.conf.presimulation_steps - 1, fetch_obstacles=False)
+            self.__presimulation_silent(max(0, self.conf.presimulation_steps-1), fetch_obstacles=False)
             if len(self.ego_vehicles) > 0:
                 self.logger.warning('<SumoSimulation/init_ego_vehicles> Ego vehicles are already defined through .rou'
                                     'file and planning problem!')
@@ -233,7 +224,7 @@ class SumoSimulation:
         self.dummy_ego_simulation = False
 
         # retrieve arbitrary route id for initialization (will not be used by interface)
-        generic_route_id = self.routedomain.getIDList()[0]
+        generic_route_id = traci.route.getIDList()[0]
 
         width = self.conf.ego_veh_width
         length = self.conf.ego_veh_length
@@ -247,7 +238,7 @@ class SumoSimulation:
                 sumo_id = EGO_ID_START + str(new_id)
                 cr_id = self._create_cr_id(type='egoVehicle', sumo_id=sumo_id, sumo_prefix=EGO_ID_START)
                 self.vehicledomain.add(sumo_id, generic_route_id, typeID="DEFAULT_VEHTYPE",
-                                       depart=None,
+                                       depart="now",
                                        departLane='first', departPos="base",
                                        departSpeed=planning_problem.initial_state.velocity,
                                        arrivalLane="current", arrivalPos="max",
@@ -259,9 +250,9 @@ class SumoSimulation:
                 position_tmp = planning_problem.initial_state.position\
                                + 0.5 * length * np.array([math.cos(planning_problem.initial_state.orientation),
                                                           math.sin(planning_problem.initial_state.orientation)])
-                self.vehicledomain.setLength(vehID=sumo_id, length=length)
-                self.vehicledomain.setWidth(vehID=sumo_id, width=width)
-                self.vehicledomain.moveToXY(vehID=sumo_id, edgeID='dummy', lane=-1,
+                self.vehicledomain.setLength(sumo_id, length=length)
+                self.vehicledomain.setWidth(sumo_id, width=width)
+                self.vehicledomain.moveToXY(sumo_id, edgeID='dummy', laneIndex=-1,
                                             x=position_tmp[0],
                                             y=position_tmp[1],
                                             angle=sumo_angle, keepRoute=2)
@@ -281,18 +272,11 @@ class SumoSimulation:
         :param ego_vehicle: the ego vehicle to be added.
         """
         if ego_vehicle.id in self._ego_vehicles:
-            self.logger.error(
+            self.logger.debug(
                 f'Ego vehicle with id {ego_vehicle.id} already exists!',
                 exc_info=True)
-
-        self._ego_vehicles[ego_vehicle.id] = ego_vehicle
-
-    @property
-    def traci_label(self):
-        """Unique label to identify simulation"""
-        if self._traci_label is None:
-            self._traci_label = self.conf.scenario_name + time.strftime("%H%M%S") + str(random.randint(0, 100))
-        return self._traci_label
+        else:
+            self._ego_vehicles[ego_vehicle.id] = ego_vehicle
 
     @property
     def ego_vehicles(self) -> Dict[int, EgoVehicle]:
@@ -441,20 +425,42 @@ class SumoSimulation:
             self._fetch_sumo_vehicles(self.current_time_step, self.vehicledomain.getIDList())
             self._fetch_sumo_pedestrians(self.current_time_step, self.persondomain.getIDList())
 
+    def _subscribe_new(self, new_ids: List[str], domain=traci.vehicle):
+        """
+        Subscribe to values of new vehicle/pedestrian
+        :param new_ids: list of new SUMO IDs
+        :param domain: domain from which should be subscribed
+        :return:
+        """
+        vehicle_ids = self.sort_ego_first(new_ids)
+        for veh_id in vehicle_ids:
+            domain.subscribe(veh_id, traci_subscription_values)
+
+    def _get_subscription_results(self, domain=traci.vehicle) -> Dict[str, Dict[int, any]]:
+        """
+        Return subscription results from current time step
+        :param domain:
+        :return: dict with results with structure {sumo_id: {value_id: value}}
+        """
+        return domain.getAllSubscriptionResults()
+
     def _fetch_sumo_vehicles(self, time_step: int, new_ids: List[str]):
         """
         Gets and stores all vehicle states from SUMO. Initializes ego vehicles when they enter simulation.
 
         """
-        vehicle_ids = self.sort_ego_first(self.vehicledomain.getIDList())
-        if len(vehicle_ids) == 0:
+        self._subscribe_new(new_ids)
+        vehicle_states = self._get_subscription_results()
+
+        if len(vehicle_states) == 0:
             return
 
         if not self.dummy_ego_simulation:
             self.check_ego_collisions(raise_error=True)
 
-        for veh_id in vehicle_ids:
-            state = self._get_current_state_from_sumo(self.vehicledomain, str(veh_id), SUMO_VEHICLE_PREFIX)
+        for veh_id, vehicle_state in vehicle_states.items():
+            state = self._extract_current_state(self.vehicledomain, veh_id, vehicle_state,
+                                                SUMO_VEHICLE_PREFIX)
             if state is None:
                 continue
 
@@ -470,6 +476,8 @@ class SumoSimulation:
 
                     if self.planning_problem_set is not None:
                         planning_problem = list(self.planning_problem_set.planning_problem_dict.values())[0]
+                    elif self.scenarios.planning_problem_set is not None:
+                        planning_problem = list(self.scenarios.planning_problem_set.planning_problem_dict.values())[0]
                     else:
                         planning_problem = None
 
@@ -492,7 +500,7 @@ class SumoSimulation:
 
             # read signal state
             if not self._silent:
-                signal_states = get_signal_state(self.vehicledomain.getSignals(veh_id), self.current_time_step)
+                signal_states = get_signal_state(vehicle_state[traci.constants.VAR_SIGNALS], self.current_time_step)
                 key = self.ids_sumo2cr['obstacleVehicle'][veh_id] \
                     if veh_id in self.ids_sumo2cr['obstacleVehicle'] else self.ids_sumo2cr[EGO_ID_START][veh_id]
                 self.signal_states[key].append(signal_states)
@@ -516,14 +524,15 @@ class SumoSimulation:
         Gets and stores all vehicle states from SUMO. Initializes ego vehicles when they enter simulation.
 
         """
-        person_ids = self.persondomain.getIDList()
+        self._subscribe_new(new_ids, domain=traci.person)
+        person_states = self._get_subscription_results(domain=traci.person)
 
-        if not person_ids:
+        if not person_states:
             return
 
         obstacle_type = "obstaclePedestrian"
-        for ped_id in person_ids:
-            state = self._get_current_state_from_sumo(self.persondomain, str(ped_id), SUMO_PEDESTRIAN_PREFIX)
+        for ped_id, ped_state in person_states.items():
+            state = self._extract_current_state(self.persondomain, ped_id, ped_state, SUMO_PEDESTRIAN_PREFIX)
             if state is None:
                 continue
             if ped_id in new_ids:
@@ -539,7 +548,7 @@ class SumoSimulation:
                 # get obstacle vehicle state
                 self.obstacle_states[time_step][self.ids_sumo2cr[obstacle_type][ped_id]] = state
 
-    def _get_veh_shape(self, vehicledomain: VehicleDomain, sumo_id):
+    def _get_veh_shape(self, vehicledomain, sumo_id):
         return Rectangle(vehicledomain.getLength(sumo_id), vehicledomain.getWidth(sumo_id))
 
     def check_ego_collisions(self, raise_error=True):
@@ -597,8 +606,8 @@ class SumoSimulation:
         finally:
             return Rectangle(length, width)
 
-    def _get_current_state_from_sumo(self, domain, veh_id: str,
-                                     sumo_prefix: str) -> Union[None, State]:
+    def _extract_current_state(self, domain, veh_id: str, vehicle_state: Dict[int, any],
+                               sumo_prefix: str) -> Union[None, State]:
         """
         Gets the current state from sumo.
         :param domain
@@ -608,8 +617,8 @@ class SumoSimulation:
         :return: the state of the given vehicle or None, if not in field of view
         """
         unique_id = sumo_prefix + veh_id
-        position = np.array(domain.getPosition(veh_id))
-        velocity = domain.getSpeed(veh_id)
+        position = np.array(vehicle_state[traci.constants.VAR_POSITION])
+        velocity = math.sqrt(vehicle_state[traci.constants.VAR_SPEED]**2 + vehicle_state[traci.constants.VAR_SPEED_LAT]**2)
 
         cr_id = sumo2cr(veh_id, self.ids_sumo2cr)
         if cr_id and cr_id in self.obstacle_shapes:
@@ -624,7 +633,7 @@ class SumoSimulation:
             delta_pos = position - self.cached_position[unique_id]
             orientation = math.atan2(delta_pos[1], delta_pos[0])
         else:
-            orientation = math.radians(-domain.getAngle(veh_id) + 90)
+            orientation = math.radians(-vehicle_state[traci.constants.VAR_ANGLE] + 90.0)
 
         self.cached_position[unique_id] = position
         position -= 0.5 * length * np.array([math.cos(orientation), math.sin(orientation)])
@@ -633,16 +642,13 @@ class SumoSimulation:
         if in_fov is False:
             return None
 
-        try:
-            acceleration = domain.getAcceleration(veh_id)
-        except AttributeError:
-            acceleration = 0
+        acceleration = vehicle_state[traci.constants.VAR_ACCELERATION]
 
-        return State(position=position,
-                     orientation=orientation,
-                     velocity=velocity,
-                     acceleration=acceleration,
-                     time_step=self.current_time_step)
+        return CustomState(position=position,
+                           orientation=orientation,
+                           velocity=velocity,
+                           acceleration=acceleration,
+                           time_step=self.current_time_step)
 
     def _get_cr_obstacles_at_time(self, time_step: int,
                                   add_ego: bool = False,
@@ -825,7 +831,7 @@ class SumoSimulation:
         Generates a new unused id for SUMO
         :return:
         """
-        id_list = self.vehicledomain.getIDList()
+        id_list = traci.vehicle.getIDList()
         new_id = int(len(id_list))
         i = 0
         while i < 1000:
@@ -1037,9 +1043,9 @@ class SumoSimulation:
             position_sumo[1] = planned_state.position[1] + len_half * math.sin(planned_state.orientation)
             sumo_angle = 90 - math.degrees(planned_state.orientation)
 
-            self.vehicledomain.moveToXY(vehID=id_sumo,
+            self.vehicledomain.moveToXY(id_sumo,
                                         edgeID='dummy',
-                                        lane=-1,
+                                        laneIndex=-1,
                                         x=position_sumo[0],
                                         y=position_sumo[1],
                                         angle=sumo_angle,
