@@ -18,7 +18,6 @@ from commonroad.scenario.trajectory import State
 from sumocr.interface.ego_vehicle import EgoVehicle
 from sumocr.interface.sumo_simulation import SumoSimulation
 from sumocr.maps.scenario_wrapper import AbstractScenarioWrapper
-from sumocr.maps.sumo_scenario import ScenarioWrapper
 from sumocr.sumo_config import DefaultConfig
 
 from sumocr.sumo_docker.rpc.messaging import get_data_chunks, save_chunks_to_file, \
@@ -28,7 +27,7 @@ from sumocr.sumo_docker.rpc.messaging import get_data_chunks, save_chunks_to_fil
 __author__ = "Peter Kocsis"
 __copyright__ = "TUM Cyber-Physical Systems Group"
 __credits__ = ["ZIM Projekt ZF4086007BZ8"]
-__version__ = "0.7.4"
+__version__ = "0.7.5"
 __maintainer__ = "Peter Kocsis"
 __email__ = "commonroad@lists.lrz.de"
 __status__ = "Integration"
@@ -57,7 +56,7 @@ class SumoRPCClient:
                 @wraps(func)
                 def logged_call(*args, **kwargs):
                     SumoRPCClient.logger.log(level, func.__qualname__ + " called")
-                    return func(*args, *kwargs)
+                    return func(*args, **kwargs)
 
                 return logged_call
 
@@ -305,7 +304,7 @@ class SumoRPCClient:
 
     def stop_server(self) -> bool:
         """Disconnect from the server
-        :return: True if the server has been succesfully stopped
+        :return: True if the server has been successfully stopped
         """
         self._request["stop_server"]()
         return self.check_connection(1, 0.5)
